#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SMPTE Time Codes"""

# pylint: disable=R0201,C0115,C0116,W0212
from fractions import Fraction
import unittest

from ttconv.time_code import SmpteTimeCode, FPS_29_97, FPS_30


class SmpteTimeCodesTest(unittest.TestCase):

  def test_parse_non_drop_frame_time_code(self):
    time_code = SmpteTimeCode.parse("01:02:03:04", FPS_30)
    self.assertEqual(time_code, SmpteTimeCode(1, 2, 3, 4, FPS_30, False))
    with self.assertRaises(ValueError):
      self.assertEqual(time_code, SmpteTimeCode(1, 2, 3, 4, FPS_30, True))
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(3, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_30, time_code.get_frame_rate())
    self.assertFalse(time_code.is_drop_frame())
    self.assertEqual(111694, time_code.to_frames())
    self.assertEqual(3600 + 2 * 60 + 3 + Fraction(4, 30), time_code.to_seconds())
    self.assertEqual("01:02:03:04", str(time_code))

  def test_parse_drop_frame_time_code(self):
    time_code = SmpteTimeCode.parse("01:02:03;04", FPS_29_97)
    self.assertEqual(time_code, SmpteTimeCode(1, 2, 3, 4, FPS_29_97, True))
    self.assertNotEqual(time_code, SmpteTimeCode(1, 2, 3, 4, FPS_29_97, False))
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(3, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_29_97, time_code.get_frame_rate())
    self.assertTrue(time_code.is_drop_frame())
    self.assertEqual(111582, time_code.to_frames())
    self.assertEqual(Fraction(111582, Fraction(30000, 1001)), time_code.to_temporal_offset())
    self.assertEqual("01:02:03;04", str(time_code))

    time_code = SmpteTimeCode.parse("01;02;03;04", FPS_29_97)
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(3, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_29_97, time_code.get_frame_rate())
    self.assertTrue(time_code.is_drop_frame())
    self.assertEqual(111582, time_code.to_frames())
    self.assertEqual(Fraction(111582, Fraction(30000, 1001)), time_code.to_temporal_offset())
    self.assertEqual("01:02:03;04", str(time_code))

    time_code = SmpteTimeCode.parse("01:02:03;04", FPS_30)
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(3, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_29_97, time_code.get_frame_rate())
    self.assertTrue(time_code.is_drop_frame())
    self.assertEqual(111582, time_code.to_frames())
    self.assertEqual(Fraction(111582, Fraction(30000, 1001)), time_code.to_temporal_offset())
    self.assertEqual("01:02:03;04", str(time_code))

    time_code = SmpteTimeCode.parse("01:02:03.04", FPS_29_97)
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(3, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_29_97, time_code.get_frame_rate())
    self.assertTrue(time_code.is_drop_frame())
    self.assertEqual(111582, time_code.to_frames())
    self.assertEqual(Fraction(111582, Fraction(30000, 1001)), time_code.to_temporal_offset())
    self.assertEqual("01:02:03;04", str(time_code))

    time_code = SmpteTimeCode.parse("01.02.03.04", FPS_29_97)
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(3, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_29_97, time_code.get_frame_rate())
    self.assertTrue(time_code.is_drop_frame())
    self.assertEqual(111582, time_code.to_frames())
    self.assertEqual(Fraction(111582, Fraction(30000, 1001)), time_code.to_temporal_offset())
    self.assertEqual("01:02:03;04", str(time_code))

  def test_time_code_frames_conversion(self):
    time_code = SmpteTimeCode.from_frames(1795, FPS_30)
    self.assertEqual("00:00:59:25", str(time_code))
    self.assertEqual(1795, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1796, FPS_30)
    self.assertEqual("00:00:59:26", str(time_code))
    self.assertEqual(1796, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1797, FPS_30)
    self.assertEqual("00:00:59:27", str(time_code))
    self.assertEqual(1797, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1798, FPS_30)
    self.assertEqual("00:00:59:28", str(time_code))
    self.assertEqual(1798, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1799, FPS_30)
    self.assertEqual("00:00:59:29", str(time_code))
    self.assertEqual(1799, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1800, FPS_30)
    self.assertEqual("00:01:00:00", str(time_code))
    self.assertEqual(1800, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1801, FPS_30)
    self.assertEqual("00:01:00:01", str(time_code))
    self.assertEqual(1801, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1802, FPS_30)
    self.assertEqual("00:01:00:02", str(time_code))
    self.assertEqual(1802, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1803, FPS_30)
    self.assertEqual("00:01:00:03", str(time_code))
    self.assertEqual(1803, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1804, FPS_30)
    self.assertEqual("00:01:00:04", str(time_code))
    self.assertEqual(1804, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1805, FPS_30)
    self.assertEqual("00:01:00:05", str(time_code))
    self.assertEqual(1805, time_code.to_frames())

    time_code = SmpteTimeCode.from_frames(17977, FPS_30)
    self.assertEqual("00:09:59:07", str(time_code))
    self.assertEqual(17977, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17978, FPS_30)
    self.assertEqual("00:09:59:08", str(time_code))
    self.assertEqual(17978, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17979, FPS_30)
    self.assertEqual("00:09:59:09", str(time_code))
    self.assertEqual(17979, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17980, FPS_30)
    self.assertEqual("00:09:59:10", str(time_code))
    self.assertEqual(17980, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17981, FPS_30)
    self.assertEqual("00:09:59:11", str(time_code))
    self.assertEqual(17981, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17982, FPS_30)
    self.assertEqual("00:09:59:12", str(time_code))
    self.assertEqual(17982, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17983, FPS_30)
    self.assertEqual("00:09:59:13", str(time_code))
    self.assertEqual(17983, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17984, FPS_30)
    self.assertEqual("00:09:59:14", str(time_code))
    self.assertEqual(17984, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17985, FPS_30)
    self.assertEqual("00:09:59:15", str(time_code))
    self.assertEqual(17985, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17986, FPS_30)
    self.assertEqual("00:09:59:16", str(time_code))
    self.assertEqual(17986, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17987, FPS_30)
    self.assertEqual("00:09:59:17", str(time_code))
    self.assertEqual(17987, time_code.to_frames())

  def test_drop_frame_time_code_frames_conversion(self):
    time_code = SmpteTimeCode.from_frames(1795, FPS_29_97)
    self.assertEqual("00:00:59;25", str(time_code))
    self.assertEqual(1795, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1796, FPS_29_97)
    self.assertEqual("00:00:59;26", str(time_code))
    self.assertEqual(1796, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1797, FPS_29_97)
    self.assertEqual("00:00:59;27", str(time_code))
    self.assertEqual(1797, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1798, FPS_29_97)
    self.assertEqual("00:00:59;28", str(time_code))
    self.assertEqual(1798, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1799, FPS_29_97)
    self.assertEqual("00:00:59;29", str(time_code))
    self.assertEqual(1799, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1800, FPS_29_97)
    self.assertEqual("00:01:00;02", str(time_code))
    self.assertEqual(1800, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1801, FPS_29_97)
    self.assertEqual("00:01:00;03", str(time_code))
    self.assertEqual(1801, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1802, FPS_29_97)
    self.assertEqual("00:01:00;04", str(time_code))
    self.assertEqual(1802, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1803, FPS_29_97)
    self.assertEqual("00:01:00;05", str(time_code))
    self.assertEqual(1803, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1804, FPS_29_97)
    self.assertEqual("00:01:00;06", str(time_code))
    self.assertEqual(1804, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(1805, FPS_29_97)
    self.assertEqual("00:01:00;07", str(time_code))
    self.assertEqual(1805, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17977, FPS_29_97)

    self.assertEqual("00:09:59;25", str(time_code))
    self.assertEqual(17977, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17978, FPS_29_97)
    self.assertEqual("00:09:59;26", str(time_code))
    self.assertEqual(17978, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17979, FPS_29_97)
    self.assertEqual("00:09:59;27", str(time_code))
    self.assertEqual(17979, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17980, FPS_29_97)
    self.assertEqual("00:09:59;28", str(time_code))
    self.assertEqual(17980, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17981, FPS_29_97)
    self.assertEqual("00:09:59;29", str(time_code))
    self.assertEqual(17981, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17982, FPS_29_97)
    self.assertEqual("00:10:00;00", str(time_code))
    self.assertEqual(17982, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17983, FPS_29_97)
    self.assertEqual("00:10:00;01", str(time_code))
    self.assertEqual(17983, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17984, FPS_29_97)
    self.assertEqual("00:10:00;02", str(time_code))
    self.assertEqual(17984, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17985, FPS_29_97)
    self.assertEqual("00:10:00;03", str(time_code))
    self.assertEqual(17985, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17986, FPS_29_97)
    self.assertEqual("00:10:00;04", str(time_code))
    self.assertEqual(17986, time_code.to_frames())
    time_code = SmpteTimeCode.from_frames(17987, FPS_29_97)
    self.assertEqual("00:10:00;05", str(time_code))
    self.assertEqual(17987, time_code.to_frames())

  def test_time_code_seconds_conversion(self):
    time_code = SmpteTimeCode.from_frames(1795, FPS_30)
    self.assertEqual("00:00:59:25", str(time_code))
    self.assertEqual(1795 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1796, FPS_30)
    self.assertEqual("00:00:59:26", str(time_code))
    self.assertEqual(1796 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1797, FPS_30)
    self.assertEqual("00:00:59:27", str(time_code))
    self.assertEqual(1797 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1798, FPS_30)
    self.assertEqual("00:00:59:28", str(time_code))
    self.assertEqual(1798 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1799, FPS_30)
    self.assertEqual("00:00:59:29", str(time_code))
    self.assertEqual(1799 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1800, FPS_30)
    self.assertEqual("00:01:00:00", str(time_code))
    self.assertEqual(1800 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1801, FPS_30)
    self.assertEqual("00:01:00:01", str(time_code))
    self.assertEqual(1801 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1802, FPS_30)
    self.assertEqual("00:01:00:02", str(time_code))
    self.assertEqual(1802 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1803, FPS_30)
    self.assertEqual("00:01:00:03", str(time_code))
    self.assertEqual(1803 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1804, FPS_30)
    self.assertEqual("00:01:00:04", str(time_code))
    self.assertEqual(1804 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1805, FPS_30)
    self.assertEqual("00:01:00:05", str(time_code))
    self.assertEqual(1805 / FPS_30, time_code.to_seconds())

    time_code = SmpteTimeCode.from_frames(17977, FPS_30)
    self.assertEqual("00:09:59:07", str(time_code))
    self.assertEqual(17977 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17978, FPS_30)
    self.assertEqual("00:09:59:08", str(time_code))
    self.assertEqual(17978 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17979, FPS_30)
    self.assertEqual("00:09:59:09", str(time_code))
    self.assertEqual(17979 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17980, FPS_30)
    self.assertEqual("00:09:59:10", str(time_code))
    self.assertEqual(17980 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17981, FPS_30)
    self.assertEqual("00:09:59:11", str(time_code))
    self.assertEqual(17981 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17982, FPS_30)
    self.assertEqual("00:09:59:12", str(time_code))
    self.assertEqual(17982 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17983, FPS_30)
    self.assertEqual("00:09:59:13", str(time_code))
    self.assertEqual(17983 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17984, FPS_30)
    self.assertEqual("00:09:59:14", str(time_code))
    self.assertEqual(17984 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17985, FPS_30)
    self.assertEqual("00:09:59:15", str(time_code))
    self.assertEqual(17985 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17986, FPS_30)
    self.assertEqual("00:09:59:16", str(time_code))
    self.assertEqual(17986 / FPS_30, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17987, FPS_30)
    self.assertEqual("00:09:59:17", str(time_code))
    self.assertEqual(17987 / FPS_30, time_code.to_seconds())

  def test_drop_frame_time_code_seconds_conversion(self):
    time_code = SmpteTimeCode.from_frames(1795, FPS_29_97)
    self.assertEqual("00:00:59;25", str(time_code))
    self.assertEqual(1795 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1796, FPS_29_97)
    self.assertEqual("00:00:59;26", str(time_code))
    self.assertEqual(1796 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1797, FPS_29_97)
    self.assertEqual("00:00:59;27", str(time_code))
    self.assertEqual(1797 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1798, FPS_29_97)
    self.assertEqual("00:00:59;28", str(time_code))
    self.assertEqual(1798 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1799, FPS_29_97)
    self.assertEqual("00:00:59;29", str(time_code))
    self.assertEqual(1799 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1800, FPS_29_97)
    self.assertEqual("00:01:00;02", str(time_code))
    self.assertEqual(1800 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1801, FPS_29_97)
    self.assertEqual("00:01:00;03", str(time_code))
    self.assertEqual(1801 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1802, FPS_29_97)
    self.assertEqual("00:01:00;04", str(time_code))
    self.assertEqual(1802 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1803, FPS_29_97)
    self.assertEqual("00:01:00;05", str(time_code))
    self.assertEqual(1803 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1804, FPS_29_97)
    self.assertEqual("00:01:00;06", str(time_code))
    self.assertEqual(1804 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(1805, FPS_29_97)
    self.assertEqual("00:01:00;07", str(time_code))
    self.assertEqual(1805 / FPS_29_97, time_code.to_seconds())

    time_code = SmpteTimeCode.from_frames(17977, FPS_29_97)
    self.assertEqual("00:09:59;25", str(time_code))
    self.assertEqual(17977 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17978, FPS_29_97)
    self.assertEqual("00:09:59;26", str(time_code))
    self.assertEqual(17978 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17979, FPS_29_97)
    self.assertEqual("00:09:59;27", str(time_code))
    self.assertEqual(17979 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17980, FPS_29_97)
    self.assertEqual("00:09:59;28", str(time_code))
    self.assertEqual(17980 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17981, FPS_29_97)
    self.assertEqual("00:09:59;29", str(time_code))
    self.assertEqual(17981 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17982, FPS_29_97)
    self.assertEqual("00:10:00;00", str(time_code))
    self.assertEqual(17982 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17983, FPS_29_97)
    self.assertEqual("00:10:00;01", str(time_code))
    self.assertEqual(17983 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17984, FPS_29_97)
    self.assertEqual("00:10:00;02", str(time_code))
    self.assertEqual(17984 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17985, FPS_29_97)
    self.assertEqual("00:10:00;03", str(time_code))
    self.assertEqual(17985 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17986, FPS_29_97)
    self.assertEqual("00:10:00;04", str(time_code))
    self.assertEqual(17986 / FPS_29_97, time_code.to_seconds())
    time_code = SmpteTimeCode.from_frames(17987, FPS_29_97)
    self.assertEqual("00:10:00;05", str(time_code))
    self.assertEqual(17987 / FPS_29_97, time_code.to_seconds())

  def test_from_frames(self):
    time_code = SmpteTimeCode.from_frames(111694, FPS_30)
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(3, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_30, time_code.get_frame_rate())
    self.assertFalse(time_code.is_drop_frame())
    self.assertEqual(111694, time_code.to_frames())
    self.assertEqual("01:02:03:04", str(time_code))
    self.assertEqual(time_code, SmpteTimeCode.from_frames(111694, FPS_30, False))
    with self.assertRaises(ValueError):
      self.assertNotEqual(time_code, SmpteTimeCode.from_frames(111694, FPS_30, True))

    time_code = SmpteTimeCode.from_frames(111694, FPS_29_97, False)
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(3, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_29_97, time_code.get_frame_rate())
    self.assertFalse(time_code.is_drop_frame())
    self.assertEqual(111694, time_code.to_frames())
    self.assertEqual("01:02:03:04", str(time_code))
    self.assertEqual(111694 / FPS_29_97, time_code.to_seconds())

    time_code = SmpteTimeCode.from_frames(111582, FPS_29_97)
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(3, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_29_97, time_code.get_frame_rate())
    self.assertTrue(time_code.is_drop_frame())
    self.assertEqual(111582, time_code.to_frames())
    self.assertEqual("01:02:03;04", str(time_code))
    self.assertEqual(time_code, SmpteTimeCode.from_frames(111582, FPS_29_97, True))
    self.assertNotEqual(time_code, SmpteTimeCode.from_frames(111582, FPS_29_97, False))

  def test_add_frames(self):
    time_code = SmpteTimeCode.parse("01:02:03:04", FPS_30)
    time_code.add_frames(30)
    self.assertEqual(1, time_code.get_hours())
    self.assertEqual(2, time_code.get_minutes())
    self.assertEqual(4, time_code.get_seconds())
    self.assertEqual(4, time_code.get_frames())
    self.assertEqual(FPS_30, time_code.get_frame_rate())
    self.assertFalse(time_code.is_drop_frame())
    self.assertEqual(111724, time_code.to_frames())
    self.assertEqual(Fraction(111724, 30), time_code.to_temporal_offset())
    self.assertEqual("01:02:04:04", str(time_code))

    time_code = SmpteTimeCode.parse("00:00:59;29", FPS_29_97)
    time_code.add_frames()
    self.assertEqual(0, time_code.get_hours())
    self.assertEqual(1, time_code.get_minutes())
    self.assertEqual(0, time_code.get_seconds())
    self.assertEqual(2, time_code.get_frames())
    self.assertEqual(FPS_29_97, time_code.get_frame_rate())
    self.assertTrue(time_code.is_drop_frame())
    self.assertEqual(1_800, time_code.to_frames())
    self.assertEqual(Fraction(1800, Fraction(30000, 1001)), time_code.to_temporal_offset())
    self.assertEqual("00:01:00;02", str(time_code))

    time_code = SmpteTimeCode.parse("00:19:59;29", FPS_29_97)
    time_code.add_frames()
    self.assertEqual(0, time_code.get_hours())
    self.assertEqual(20, time_code.get_minutes())
    self.assertEqual(0, time_code.get_seconds())
    self.assertEqual(0, time_code.get_frames())
    self.assertEqual(FPS_29_97, time_code.get_frame_rate())
    self.assertTrue(time_code.is_drop_frame())
    self.assertEqual(35_964, time_code.to_frames())
    self.assertEqual(Fraction(35964, Fraction(30000, 1001)), time_code.to_temporal_offset())
    self.assertEqual("00:20:00;00", str(time_code))

if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SCC lines"""

# pylint: disable=R0201,C0115,C0116,W0212
import unittest

from ttconv.scc.line import SccLine
from ttconv.scc.caption_style import SccCaptionStyle
from ttconv.time_code import SmpteTimeCode, FPS_30


class SccLineTest(unittest.TestCase):

  def test_scc_line_from_str_with_pop_on_style(self):
    line_str = "01:03:27:29	94ae 94ae 9420 9420 94f2 94f2 c845 d92c 2054 c845 5245 ae80 942c 942c 8080 8080 942f 942f"
    scc_line = SccLine.from_str(line_str)
    self.assertEqual(18, len(scc_line.scc_words))
    self.assertEqual(SmpteTimeCode(1, 3, 27, 29, FPS_30).to_temporal_offset(), scc_line.time_code.to_temporal_offset())
    self.assertEqual("01:03:27:29	{ENM}{ENM}{RCL}{RCL}{1504}{1504}HEY, THERE.{EDM}{EDM}{}{}{EOC}{EOC}", scc_line.to_disassembly())
    self.assertEqual(SccCaptionStyle.PopOn, scc_line.get_style())

  def test_scc_line_from_invalid_str(self):
    self.assertIsNone(SccLine.from_str(""))
    self.assertIsNone(SccLine.from_str("Hello world!"))

  def test_scc_line_from_str_with_unknown_style(self):
    line_str = "01:03:27:29	"
    scc_line = SccLine.from_str(line_str)
    self.assertEqual(0, len(scc_line.scc_words))
    self.assertEqual(SmpteTimeCode(1, 3, 27, 29, FPS_30).to_temporal_offset(), scc_line.time_code.to_temporal_offset())
    self.assertEqual("01:03:27:29	", scc_line.to_disassembly())
    self.assertEqual(SccCaptionStyle.Unknown, scc_line.get_style())

    line_str = "01:03:27:29	9024 c845 d92c 2054 c845 5245 ae80 9f9f"
    scc_line = SccLine.from_str(line_str)
    self.assertEqual(8, len(scc_line.scc_words))
    self.assertEqual(SmpteTimeCode(1, 3, 27, 29, FPS_30).to_temporal_offset(), scc_line.time_code.to_temporal_offset())
    self.assertEqual("01:03:27:29	{BBl}HEY, THERE.{??}", scc_line.to_disassembly())
    self.assertEqual(SccCaptionStyle.Unknown, scc_line.get_style())

  def test_scc_line_from_str_with_roll_up_style(self):
    line_str = "01:03:27:29	9425 9425 94ad 94ad 94c8 94c8 c845 d92c 2054 c845 5245 ae80"
    scc_line = SccLine.from_str(line_str)
    self.assertEqual(12, len(scc_line.scc_words))
    self.assertEqual(SmpteTimeCode(1, 3, 27, 29, FPS_30).to_temporal_offset(), scc_line.time_code.to_temporal_offset())
    self.assertEqual("01:03:27:29	{RU2}{RU2}{CR}{CR}{14R}{14R}HEY, THERE.", scc_line.to_disassembly())
    self.assertEqual(SccCaptionStyle.RollUp, scc_line.get_style())

  def test_scc_line_from_str_with_paint_on_style(self):
    line_str = "01:03:27:29	9429 9429 94f2 94f2 c845 d92c 2054 c845 5245 ae80"
    scc_line = SccLine.from_str(line_str)
    self.assertEqual(10, len(scc_line.scc_words))
    self.assertEqual(SmpteTimeCode(1, 3, 27, 29, FPS_30).to_temporal_offset(), scc_line.time_code.to_temporal_offset())
    self.assertEqual("01:03:27:29	{RDC}{RDC}{1504}{1504}HEY, THERE.", scc_line.to_disassembly())
    self.assertEqual(SccCaptionStyle.PaintOn, scc_line.get_style())



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2021, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''Unit tests for ISO 6937 decoder'''

# pylint: disable=R0201,C0115,C0116

import unittest
from ttconv.stl import iso6937

class ISO6937Tests(unittest.TestCase):

  def test_valid_one_char_decode(self):
    self.assertEqual(iso6937.decode(b'\xdc')[0], '\u215B')

  def test_valid_two_chars_decode(self):
    self.assertEqual(iso6937.decode(b'\xc1A')[0], '\u00C0')

  def test_invalid_one_char_decode(self):
    self.assertEqual(iso6937.decode(b'\x00', "replace")[0], '\uFFFD')
  
  def test_invalid_two_chars_decode(self):
    self.assertEqual(iso6937.decode(b'\xcdz', "replace")[0], '\uFFFD')

if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Filter for style properties supported by the output"""

import logging
from typing import Dict, List, Type

from ttconv.filters.isd_filter import ISDFilter
from ttconv.isd import ISD
from ttconv.model import ContentElement
from ttconv.style_properties import StyleProperty
import ttconv.filters.supported_style_properties

LOGGER = logging.getLogger(__name__)


class SupportedStylePropertiesISDFilter(ISDFilter):
  """Filter that remove unsupported style properties"""

  def __init__(self, supported_style_properties: Dict[Type[StyleProperty], List]):
    self.filter = ttconv.filters.supported_style_properties.SupportedStylePropertiesFilter(supported_style_properties)

  def process(self, isd: ISD):
    """Filter ISD document style properties"""
    LOGGER.debug("Filter default style properties from ISD.")

    for region in isd.iter_regions():
      self.filter.process_element(region)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2022, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""WebVTT Cue Text tokenizer. See https://www.w3.org/TR/webvtt1/#webvtt-cue-text-tokenizer"""

from __future__ import annotations
from enum import Enum
import re
from typing import List, Optional
import html

class StringBuf:
  """Simple class that allows a string to be built from the concatenation of
  multiple strings, starting with an initial string."""
  def __init__(self, initial_value: str = None) -> None:
    self.buf: List[str] = [initial_value] if initial_value is not None else []

  def append(self, s: str) -> None:
    self.buf.append(s)

  def extend(self, sb: StringBuf) -> None:
    self.buf.extend(sb.buf)

  def __str__(self) -> str:
    return "".join(self.buf)

  def is_empty(self) -> bool:
    return len(self.buf) == 0 or all(map(lambda x: len(x) == 0, self.buf))

class Token:
  """Base class for WebVTT Cue Text tokens."""
  pass

class StringToken(Token):
  """WebVTT Cue Text string token."""
  def __init__(self, value: str = "") -> None:
    super().__init__()
    self.value = value

class StartTagToken(Token):
  """WebVTT Cue Text start tag token"""
  def __init__(self, tag: str = "", classes: Optional[List[str]] = None, annotation: Optional[str] = None) -> None:
    super().__init__()
    self.tag = tag
    self.classes = classes
    self.annotation = annotation

class EndTagToken(Token):
  """WebVTT Cue Text end tag token"""
  def __init__(self, tag: str = "") -> None:
    super().__init__()
    self.tag = tag

class TimestampTagToken(Token):
  """WebVTT Cue Text timestamp tag token"""
  def __init__(self, timestamp: str = "") -> None:
    super().__init__()
    self.timestamp = timestamp


def CueTextTokenizer(cue_text: str):
  """Generator that outputs a sequence of WebVTT Cue Text tokens from a WebVTT
  Cue Text."""
  EOF_MARKER = -1

  class _State(Enum):
    data = 1
    tag = 2
    data_cref = 3
    start_tag = 4
    start_tag_annot = 5
    start_tag_class = 6
    end_tag = 7
    ts_tag = 8
    annot_cref = 3

  cue_text: str = cue_text
  position: int = 0

  # token loop
  while position < len(cue_text):
    state: _State = _State.data
    result: StringBuf = StringBuf()
    classes: List[str] = []
    buffer: StringBuf = StringBuf()

    # codepoint loop
    while True:
      c = ord(cue_text[position]) if position < len(cue_text) else EOF_MARKER

      if state is _State.data:
        if c == ord("&"):
          state = _State.data_cref
          buffer = StringBuf("&")
        elif c == ord("<"):
          if result.is_empty():
            state = _State.tag
          else:
            yield StringToken(str(result))
            break
        elif c == EOF_MARKER:
          yield StringToken(str(result))
          break
        else:
          result.append(chr(c))

      elif state is _State.data_cref:
        if c == ord(";"):
          coded_entity = str(buffer)
          decoded_entity = html.unescape(coded_entity)
          if decoded_entity == coded_entity :
            result.extend(buffer)
          else:
            result.append(decoded_entity)
          state = _State.data
        elif c == EOF_MARKER:
          result.extend(buffer)
          state = _State.data
          continue
        else:
          buffer.append(chr(c))

      elif state is _State.tag:
        if c in (0x09, 0x0A, 0x0C, 0x20):
          state = _State.start_tag_annot
        elif c == ord("."):
          state = _State.start_tag_class
        elif c == ord("/"):
          state = _State.end_tag
        elif ord("0") <= c <= ord("9"):
          result = StringBuf(chr(c))
          state = _State.ts_tag
        elif c in (ord(">"), EOF_MARKER):
          if c == ord(">"):
            position += 1
          yield StringToken()
          break
        else:
          result = StringBuf(chr(c))
          state = _State.start_tag

      elif state is _State.start_tag:
        if c in (0x09, 0x0C, 0x20):
          state = _State.start_tag_annot
        elif c == 0x0A:
          buffer = StringBuf(chr(c))
          state = _State.start_tag_annot
        elif c is ord("."):
          state = _State.start_tag_class
        elif c in (ord(">"), EOF_MARKER):
          if c == ord(">"):
            position += 1
          yield StartTagToken(str(result))
          break
        else:
          result.append(chr(c))

      elif state is _State.start_tag_class:
        if c in (0x09, 0x0C, 0x20):
          classes.append(str(buffer))
          buffer = StringBuf()
          state = _State.start_tag_annot
        elif c == 0x0A:
          classes.append(str(buffer))
          buffer = StringBuf(chr(c))
          state = _State.start_tag_annot
        elif c is ord("."):
          classes.append(str(buffer))
          buffer = StringBuf()
        elif c in (EOF_MARKER, ord(">")):
          if c == ord(">"):
            position += 1
          classes.append(str(buffer))
          yield StartTagToken(str(result), classes)
          break
        else:
          buffer.append(chr(c))

      elif state is _State.start_tag_annot:

        if c == ord("&"):
          state = _State.annot_cref
          buffer = StringBuf("&")
        elif c in (ord(">"), EOF_MARKER):
          if c == ord(">"):
            position += 1
          annot = re.sub(r"\s+", " ", str(buffer).strip())
          yield StartTagToken(str(result), classes, annot)
          break
        else:
          buffer.append(chr(c))

      elif state is _State.annot_cref:
        if c == ord(";"):
          coded_entity = str(buffer)
          decoded_entity = html.unescape(coded_entity)
          if decoded_entity == coded_entity:
            result.extend(buffer)
          else:
            result.append(decoded_entity)
          state = _State.start_tag_annot
        elif c in (EOF_MARKER, ord(">")):
          result.extend(buffer)
          state = _State.start_tag_annot
          continue
        else:
          buffer.append(chr(c))

      elif state is _State.end_tag:
        if c in (ord(">"), EOF_MARKER):
          if c == ord(">"):
            position += 1
          yield EndTagToken(str(result))
          break
        result.append(chr(c))

      elif state is _State.ts_tag:
        if c in (ord(">"), EOF_MARKER):
          if c == ord(">"):
            position += 1
          yield TimestampTagToken(str(result))
          break
        result.append(chr(c))

      else:
        raise RuntimeError("Bad state")

      position += 1



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''IMSC writer'''

import logging
import numbers
import typing
import xml.etree.ElementTree as et
import ttconv.imsc.elements as imsc_elements
import ttconv.imsc.namespaces as xml_ns
import ttconv.model as model
import ttconv.imsc.config as imsc_config
from ttconv.imsc.attributes import TimeExpressionSyntaxEnum

LOGGER = logging.getLogger(__name__)

#
# imsc writer
#

def from_model(
  model_doc: model.ContentDocument,
  config: typing.Optional[imsc_config.IMSCWriterConfiguration] = None,
  progress_callback: typing.Callable[[numbers.Real], typing.NoReturn] = lambda _: None
  ):
  '''Converts the data model to an IMSC document. The writer regularly the `progress_callback` function, if provided,
  with a real between 0 and 1, indicating the relative progress of the process.
  '''
  
  et.register_namespace("", xml_ns.TTML)
  et.register_namespace("ttp", xml_ns.TTP)
  et.register_namespace("tts", xml_ns.TTS)
  et.register_namespace("ittp", xml_ns.ITTP)
  et.register_namespace("itts", xml_ns.ITTS)
  et.register_namespace("ebutts", xml_ns.EBUTTS)

  if config is None:
    config = imsc_config.IMSCWriterConfiguration()

  if config.time_format is not None:

    if config.time_format in (TimeExpressionSyntaxEnum.clock_time_with_frames, TimeExpressionSyntaxEnum.frames):

      if config.fps is None:
        raise ValueError("HH:MM:SS:FF and frames time expressions require the `frame_rate` parameter to be set.")

      if config.time_format is TimeExpressionSyntaxEnum.clock_time_with_frames and config.fps.denominator != 1:
        raise ValueError("Time expressions cannot be HH:MM:SS:FF if the `frame_rate` parameter is not an integer")

    time_format = config.time_format

  elif config.fps is not None:
    time_format = TimeExpressionSyntaxEnum.frames

  else:
    time_format = TimeExpressionSyntaxEnum.clock_time

  return et.ElementTree(
    imsc_elements.TTElement.from_model(
      model_doc,
      config.fps,
      time_format,
      progress_callback,
      config.profile_signaling
    )
  )



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SCC content"""

# pylint: disable=R0201,C0115,C0116,W0212

import unittest

from ttconv.scc.caption_line import SccCaptionLine
from ttconv.scc.caption_text import SccCaptionText
from ttconv.style_properties import StyleProperties, NamedColors


class SccCaptionLineTest(unittest.TestCase):

  def test_line(self):
    row = 6
    indent = 2
    caption_line = SccCaptionLine(row, indent)

    self.assertEqual(row, caption_line.get_row())
    self.assertEqual(indent, caption_line.get_indent())

    caption_line.set_row(7)
    caption_line.indent(2)

    self.assertEqual(7, caption_line.get_row())
    self.assertEqual(4, caption_line.get_indent())

    self.assertTrue(caption_line.is_empty())
    self.assertEqual(0, caption_line.get_cursor())
    self.assertEqual(0, caption_line.get_length())
    self.assertEqual(1, len(caption_line.get_texts()))
    self.assertTrue(caption_line.get_current_text().is_empty())

    caption_line.set_cursor(10)
    self.assertEqual(0, caption_line.get_cursor())

    caption_line.add_text("Hello ")
    caption_text = caption_line.get_current_text()

    self.assertIsNotNone(caption_text)
    self.assertEqual("Hello ", caption_text.get_text())
    self.assertEqual(6, caption_text.get_cursor())
    self.assertEqual(6, caption_text.get_length())

    self.assertEqual(6, caption_line.get_cursor())
    self.assertEqual(6, caption_line.get_length())
    self.assertListEqual([caption_text], caption_line.get_texts())

    another_caption_text = SccCaptionText("World!")

    caption_line.add_text(another_caption_text)

    self.assertEqual(another_caption_text, caption_line.get_current_text())
    self.assertEqual("World!", another_caption_text.get_text())
    self.assertEqual(6, another_caption_text.get_cursor())
    self.assertEqual(6, another_caption_text.get_length())

    self.assertEqual(12, caption_line.get_cursor())
    self.assertEqual(12, caption_line.get_length())
    self.assertListEqual([caption_text, another_caption_text], caption_line.get_texts())

    caption_line.set_cursor(6)
    self.assertEqual(another_caption_text, caption_line.get_current_text())

    caption_line.add_text("hello")

    self.assertEqual("hello!", another_caption_text.get_text())
    self.assertEqual(5, another_caption_text.get_cursor())
    self.assertEqual(6, another_caption_text.get_length())

    self.assertEqual(11, caption_line.get_cursor())
    self.assertEqual(12, caption_line.get_length())
    self.assertListEqual([caption_text, another_caption_text], caption_line.get_texts())

    caption_line.set_cursor(5)
    self.assertEqual(caption_text, caption_line.get_current_text())
    self.assertEqual(5, caption_text.get_cursor())

    caption_line.add_text(", abcd")

    self.assertEqual("Hello,", caption_text.get_text())
    self.assertEqual(6, caption_text.get_cursor())
    self.assertEqual(6, caption_text.get_length())

    self.assertEqual(another_caption_text, caption_line.get_current_text())
    self.assertEqual(" abcd!", another_caption_text.get_text())
    self.assertEqual(5, another_caption_text.get_cursor())
    self.assertEqual(6, another_caption_text.get_length())

    self.assertEqual(11, caption_line.get_cursor())
    self.assertEqual(12, caption_line.get_length())
    self.assertListEqual([caption_text, another_caption_text], caption_line.get_texts())

    caption_line.set_cursor(7)
    self.assertEqual(another_caption_text, caption_line.get_current_text())
    self.assertEqual(1, another_caption_text.get_cursor())

    caption_line.add_text("123456789")

    self.assertEqual(another_caption_text, caption_line.get_current_text())
    self.assertEqual(" 123456789", another_caption_text.get_text())
    self.assertEqual(10, another_caption_text.get_cursor())
    self.assertEqual(10, another_caption_text.get_length())

    self.assertEqual(16, caption_line.get_cursor())
    self.assertEqual(16, caption_line.get_length())
    self.assertListEqual([caption_text, another_caption_text], caption_line.get_texts())

    caption_line.clear()
    self.assertEqual(0, caption_line.get_cursor())
    self.assertEqual(0, caption_line.get_length())
    self.assertTrue(caption_line.is_empty())

  def test_line_text_leading_and_trailing_spaces(self):
    line = SccCaptionLine(0, 0)
    line.add_text("123")
    self.assertEqual(0, line.get_leading_spaces())
    self.assertEqual(0, line.get_trailing_spaces())

    line = SccCaptionLine(0, 0)
    line.add_text("   123")
    self.assertEqual(3, line.get_leading_spaces())
    self.assertEqual(0, line.get_trailing_spaces())

    line = SccCaptionLine(0, 0)
    line.add_text("123   ")
    self.assertEqual(0, line.get_leading_spaces())
    self.assertEqual(3, line.get_trailing_spaces())

    line = SccCaptionLine(0, 0)
    line.add_text(" 123  ")
    self.assertEqual(1, line.get_leading_spaces())
    self.assertEqual(2, line.get_trailing_spaces())

    line = SccCaptionLine(0, 0)
    line.add_text("   ")
    line.add_text(SccCaptionText(" 123 "))
    line.add_text(SccCaptionText(" "))
    line.add_text(SccCaptionText("  "))
    self.assertEqual(4, line.get_leading_spaces())
    self.assertEqual(4, line.get_trailing_spaces())


class SccCaptionTextTest(unittest.TestCase):

  def test_text_insertion(self):
    caption_text = SccCaptionText()
    self.assertEqual(0, caption_text.get_cursor())
    caption_text.append("Lorem ")
    self.assertEqual(6, caption_text.get_cursor())
    self.assertEqual("Lorem ", caption_text.get_text())

    caption_text.append("ipsum")
    self.assertEqual(11, caption_text.get_cursor())
    self.assertEqual("Lorem ipsum", caption_text.get_text())

    caption_text.set_cursor_at(0)
    self.assertEqual(0, caption_text.get_cursor())
    self.assertEqual("Lorem ipsum", caption_text.get_text())

    caption_text.append("Hello")
    self.assertEqual(5, caption_text.get_cursor())
    self.assertEqual("Hello ipsum", caption_text.get_text())

    caption_text.set_cursor_at(6)
    caption_text.append("World!")
    self.assertEqual(12, caption_text.get_cursor())
    self.assertEqual("Hello World!", caption_text.get_text())

    caption_text.set_cursor_at(5)
    caption_text.append("! Abc")
    self.assertEqual(10, caption_text.get_cursor())
    self.assertEqual("Hello! Abcd!", caption_text.get_text())

  def test_style_properties(self):
    caption_text = SccCaptionText()

    caption_text.add_style_property(StyleProperties.Color, None)
    self.assertEqual(0, len(caption_text.get_style_properties()))

    caption_text.add_style_property(StyleProperties.Color, NamedColors.fuchsia.value)
    self.assertEqual(1, len(caption_text.get_style_properties()))
    self.assertEqual(NamedColors.fuchsia.value, caption_text.get_style_properties()[StyleProperties.Color])

    other_caption_text = SccCaptionText()
    self.assertFalse(caption_text.has_same_style_properties(other_caption_text))

    other_caption_text.add_style_property(StyleProperties.Color, NamedColors.fuchsia.value)
    self.assertTrue(caption_text.has_same_style_properties(other_caption_text))


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""SCC caption paragraph"""

import copy
import logging
import math
from math import ceil
from typing import Optional, List, Dict, Union

from ttconv.model import Region, ContentDocument, P, Br, Span, Text
from ttconv.scc.caption_line import SccCaptionLine
from ttconv.scc.caption_style import SccCaptionStyle
from ttconv.scc.caption_text import SccCaptionText
from ttconv.scc.utils import get_position_from_offsets, get_extent_from_dimensions, convert_cells_to_percentages
from ttconv.style_properties import CoordinateType, ExtentType, StyleProperties, LengthType, DisplayAlignType, ShowBackgroundType, \
  TextAlignType, NamedColors
from ttconv.time_code import SmpteTimeCode

LOGGER = logging.getLogger(__name__)

SCC_SAFE_AREA_CELL_RESOLUTION_ROWS = 15
SCC_SAFE_AREA_CELL_RESOLUTION_COLUMNS = 32

SCC_ROOT_CELL_RESOLUTION_ROWS = ceil(SCC_SAFE_AREA_CELL_RESOLUTION_ROWS / 0.80)
SCC_ROOT_CELL_RESOLUTION_COLUMNS = ceil(SCC_SAFE_AREA_CELL_RESOLUTION_COLUMNS / 0.80)


class SccCaptionParagraph:
  """Caption paragraph"""

  @staticmethod
  def default(caption_style: SccCaptionStyle = SccCaptionStyle.Unknown):
    """Initializes a default caption paragraph"""
    return SccCaptionParagraph(caption_style=caption_style)

  def __init__(self, safe_area_x_offset: int = 0, safe_area_y_offset: int = 0,
               caption_style: SccCaptionStyle = SccCaptionStyle.Unknown):
    self._caption_id: str = ""
    self._begin: Optional[SmpteTimeCode] = None
    self._end: Optional[SmpteTimeCode] = None
    self._safe_area_x_offset = safe_area_x_offset
    self._safe_area_y_offset = safe_area_y_offset

    # Paragraph offsets (= origin) in the root area
    self._column_offset: int = safe_area_x_offset
    self._row_offset: int = safe_area_y_offset

    # Position of the cursor in the active area
    self._cursor: (int, int) = (0, 0)
    # Line where the cursor is currently positioned
    self._current_line: Optional[SccCaptionLine] = None
    # Lines per row in the active area (will be separated by line-breaks)
    self._caption_lines: Dict[int, SccCaptionLine] = {}
    # Initialize first default line
    self.new_caption_line()

    self._caption_style: SccCaptionStyle = caption_style
    self._style_properties = {}

  def set_id(self, caption_id: str):
    """Sets caption identifier"""
    self._caption_id = caption_id

  def get_id(self) -> str:
    """Returns the caption identifier"""
    return self._caption_id

  def set_begin(self, time_code):
    """Sets caption begin time code"""
    self._begin = copy.copy(time_code)

  def get_begin(self) -> Optional[SmpteTimeCode]:
    """Returns the caption begin time code"""
    return self._begin

  def set_end(self, time_code):
    """Sets caption end time code"""
    self._end = copy.copy(time_code)

  def get_end(self) -> Optional[SmpteTimeCode]:
    """Returns the caption end time code"""
    return self._end

  def get_safe_area_x_offset(self):
    """Returns the safe area x offset"""
    return self._safe_area_x_offset

  def get_safe_area_y_offset(self):
    """Returns the safe area y offset"""
    return self._safe_area_y_offset

  def set_caption_style(self, caption_style: SccCaptionStyle):
    """Sets the caption style"""
    self._caption_style = caption_style

  def get_caption_style(self) -> SccCaptionStyle:
    """Returns the caption style"""
    return self._caption_style

  def get_current_line(self) -> SccCaptionLine:
    """Returns the current caption line"""
    return self._current_line

  def get_current_text(self) -> SccCaptionText:
    """Returns the current caption text"""
    return self._current_line.get_current_text()

  def append_text(self, text: str):
    """Append text to current line text content"""
    self.get_current_line().add_text(text)
    self.indent_cursor(len(text))

  def set_lines(self, lines: Union[List[SccCaptionLine], Dict[int, SccCaptionLine]]):
    """Sets paragraph lines"""
    if isinstance(lines, dict):
      self._caption_lines = lines
      return

    for line in lines:
      self._caption_lines[line.get_row()] = line

  def get_style_properties(self) -> dict:
    """Sets the style properties map"""
    return self._style_properties

  def add_style_property(self, style_property, value):
    """Adds a style property"""
    if value is None:
      return
    self._style_properties[style_property] = value

  def get_style_property(self, style_property) -> Optional:
    """Returns the style property value"""
    return self._style_properties.get(style_property)

  def set_cursor_at(self, row: int, indent: Optional[int] = None):
    """Set cursor position and initialize a new line if necessary"""

    if self._caption_lines.get(self._current_line.get_row()) is not None:
      # Set current line if necessary
      if self._caption_lines.get(self._current_line.get_row()) is not self._current_line:
        self._current_line = self._caption_lines.get(self._current_line.get_row())

      # Remove current line if empty (i.e. useless)
      if self._current_line.is_empty():
        del self._caption_lines[self._current_line.get_row()]

    self._cursor = (row, indent if indent is not None else 0)

    if self._caption_lines.get(row) is None:
      self.new_caption_line()

    self._current_line = self._caption_lines.get(row)

    if indent is not None:
      self._update_current_line_cursor()

  def get_cursor(self) -> (int, int):
    """Returns cursor coordinates"""
    return self._cursor

  def indent_cursor(self, indent: int):
    """Set cursor position of the current row"""
    self._cursor = (self._cursor[0], self._cursor[1] + indent)

    if self._current_line.is_empty():
      # If the current line is empty, set cursor indent as a line tabulation
      self._current_line.indent(indent)
    else:
      self._update_current_line_cursor()

  def _update_current_line_cursor(self):
    """Updates cursor position on current line"""
    new_cursor_position = self._cursor[1] - self._current_line.get_indent()

    if new_cursor_position < 0:
      self._current_line.indent(new_cursor_position)

    self._current_line.set_cursor(new_cursor_position)

  def get_lines(self) -> Dict[int, SccCaptionLine]:
    """Returns the paragraph lines per row"""
    return self._caption_lines

  def is_empty(self) -> bool:
    """Returns whether the paragraph has no content"""
    return self._get_length() == 0

  def _get_length(self) -> int:
    """Returns the total length of contained text"""
    return sum([line.get_length() for line in self._caption_lines.values()])

  def copy_lines(self) -> Dict[int, SccCaptionLine]:
    """Copy paragraph lines (without time attributes)"""
    lines_copy = {}
    for row, orig_line in self._caption_lines.items():
      new_line = SccCaptionLine(orig_line.get_row(), orig_line.get_indent())

      for orig_text in orig_line.get_texts():
        new_text = SccCaptionText(orig_text.get_text())
        for style_type, style_value in orig_text.get_style_properties().items():
          new_text.add_style_property(style_type, style_value)
        new_line.add_text(new_text)
      lines_copy[row] = new_line

    return lines_copy

  def new_caption_text(self):
    """Appends a new caption text content, and keeps reference on it"""
    self._current_line.add_text(SccCaptionText())

  def new_caption_line(self):
    """Appends a new caption text content, and keeps reference on it"""
    self._caption_lines[self._cursor[0]] = SccCaptionLine(self._cursor[0], self._cursor[1])
    self._current_line = self._caption_lines[self._cursor[0]]

  def roll_up(self):
    """Set specified lines on a roll-up configuration"""
    if self._caption_style is not SccCaptionStyle.RollUp:
      raise RuntimeError(f"Cannot roll-Up {self._caption_style.name}-styled caption.")

    rows_to_roll_up = list(sorted(self._caption_lines.keys()))

    # Roll each line one row up
    for row in rows_to_roll_up:
      line = self._caption_lines.pop(row)
      if row == 0:
        continue
      line.set_row(row - 1)
      self._caption_lines[line.get_row()] = line

  def get_origin(self) -> CoordinateType:
    """Computes and returns the current paragraph origin, based on its content"""
    if not self.is_empty():
      x_offsets = [text.get_indent() for text in self._caption_lines.values()]
      y_offsets = [text.get_row() - 1 for text in self._caption_lines.values()]

      return get_position_from_offsets(min(x_offsets) + self._safe_area_x_offset, min(y_offsets) + self._safe_area_y_offset)

    return get_position_from_offsets(self._safe_area_x_offset, self._safe_area_y_offset)

  def get_extent(self) -> ExtentType:
    """Computes and returns the current paragraph extent, based on its content"""
    if self.is_empty():
      return get_extent_from_dimensions(0, 0)

    paragraph_rows = self._caption_lines.keys()
    nb_lines = max(paragraph_rows) - min(paragraph_rows) + 1
    max_text_lengths = max([line.get_length() for line in self._caption_lines.values()])

    return get_extent_from_dimensions(max_text_lengths, nb_lines)

  def get_last_caption_lines(self, expected_lines: int) -> List[SccCaptionLine]:
    """Returns the caption text elements from the expected number of last lines"""
    if expected_lines <= 0:
      return []

    sorted_lines = list(dict(sorted(self._caption_lines.items())).values())
    return sorted_lines[-expected_lines:]

  def guess_text_alignment(self) -> TextAlignType:
    """Tries to detect the text alignment according to the content indentation"""

    def get_line_right_offset(line: SccCaptionLine) -> int:
      return SCC_ROOT_CELL_RESOLUTION_COLUMNS - (line.get_indent() + line.get_length())

    if self.is_empty():
      return TextAlignType.start

    # look for longest line
    longest_line = max(self._caption_lines.values(), key=lambda line: line.get_length())

    # define borders based on the longest line
    left_border = longest_line.get_indent() + longest_line.get_leading_spaces()
    right_border = get_line_right_offset(longest_line) + longest_line.get_trailing_spaces()

    # is the text left-aligned?
    if all(l.get_indent() + l.get_leading_spaces() - left_border == 0 for l in self._caption_lines.values()):
      return TextAlignType.start

    # is the text right-aligned?
    if all(get_line_right_offset(l) + l.get_trailing_spaces() - right_border == 0 for l in self._caption_lines.values()):
      return TextAlignType.end

    # is the text centered?
    if all(abs(l.get_indent() + l.get_leading_spaces() - left_border - (
        get_line_right_offset(l) + l.get_trailing_spaces() - right_border)) < 2 for l in self._caption_lines.values()):
      return TextAlignType.center

    # default is left-aligned
    LOGGER.warning("Cannot define the paragraph text alignment. Set it to default left-aligned.")
    return TextAlignType.start

  def to_paragraph(self, doc: ContentDocument) -> P:
    """Converts and returns current caption paragraph into P instance"""

    # Set up a new paragraph
    p = P()
    p.set_doc(doc)
    p.set_id(self._caption_id)

    if self._begin is not None:
      p.set_begin(self._begin.to_temporal_offset())
    if self._end is not None:
      p.set_end(self._end.to_temporal_offset())

    # Set the region to current caption
    region = _SccParagraphRegion(self, doc)
    p.set_region(region.get_region())

    # Set the paragraph style
    for (prop, value) in self.get_style_properties().items():
      p.set_style(prop, value)

    # Add caption content (text and line-breaks)
    last_row: Optional[int] = None
    for row, caption_line in sorted(self._caption_lines.items()):

      if last_row is not None:
        for _ in range(0, abs(last_row - row)):
          p.push_child(Br(doc))

      last_row = row

      for caption_text in caption_line.get_texts():

        # Skip empty texts
        if caption_text.is_empty():
          continue

        span = Span(doc)

        if caption_text.get_begin() is not None:

          begin = caption_text.get_begin().to_temporal_offset()

          if self.get_caption_style() is SccCaptionStyle.PaintOn:
            # Compute paragraph-relative begin time
            begin -= self._begin.to_temporal_offset()

          span.set_begin(begin)

        if caption_text.get_end() is not None:

          end = caption_text.get_end().to_temporal_offset()

          if self.get_caption_style() is SccCaptionStyle.PaintOn:
            # Compute paragraph-relative end time
            end -= self._end.to_temporal_offset()

          span.set_end(end)

        for (prop, value) in caption_text.get_style_properties().items():
          span.set_style(prop, value)

        if StyleProperties.BackgroundColor not in caption_text.get_style_properties():
          span.set_style(StyleProperties.BackgroundColor, NamedColors.black.value)

        span.push_child(Text(doc, caption_text.get_text()))

        p.push_child(span)

    return p

  def __repr__(self) -> str:
    return "<" + self.__class__.__name__ + " " + str(self.__dict__) + ">"


class _SccParagraphRegion:
  """SCC paragraph region utility class"""

  def __init__(self, paragraph: SccCaptionParagraph, doc: ContentDocument):
    self._paragraph = paragraph
    self._doc = doc

    cell_resolution = doc.get_cell_resolution()
    x_offset = self._paragraph.get_safe_area_x_offset()
    y_offset = self._paragraph.get_safe_area_y_offset()

    self._left = x_offset
    self._top = y_offset
    self._right = cell_resolution.columns - x_offset
    # Add 1 cell to bottom since the cursor height is 1
    self._bottom = cell_resolution.rows - y_offset + 1

  def get_region(self) -> Region:
    """Get a paragraph matching region"""

    matching_region = self._find_matching_region()

    if matching_region is None:
      # Create a new matching region
      matching_region = self._create_matching_region()
    else:
      # Extend region to paragraph if needed
      self._extend_region_to_paragraph(matching_region)

    return matching_region

  def _has_same_origin_as_region(self, region: Region) -> bool:
    """Checks whether the region origin is the same as the paragraph origin"""
    region_origin: Optional[CoordinateType] = region.get_style(StyleProperties.Origin)

    # Convert paragraph origin units into percentages
    paragraph_origin = convert_cells_to_percentages(self._paragraph.get_origin(), self._doc.get_cell_resolution())

    if self._paragraph.get_caption_style() in (SccCaptionStyle.RollUp, SccCaptionStyle.PaintOn):
      return region_origin \
             and region_origin.x.units is paragraph_origin.x.units \
             and region_origin.y.units is paragraph_origin.y.units \
             and math.isclose(region_origin.x.value, paragraph_origin.x.value, abs_tol=0.001) \
             and region_origin.y.value <= paragraph_origin.y.value

    return region_origin \
           and region_origin.x.units is paragraph_origin.x.units \
           and region_origin.y.units is paragraph_origin.y.units \
           and math.isclose(region_origin.x.value, paragraph_origin.x.value, abs_tol=0.001) \
           and math.isclose(region_origin.y.value, paragraph_origin.y.value, abs_tol=0.001)

  def _find_matching_region(self) -> Optional[Region]:
    """Looks for a region that origin matches with the paragraph origin"""
    for region in self._doc.iter_regions():
      if self._has_same_origin_as_region(region) and region.get_id().startswith(self._get_region_prefix()):
        return region
    return None

  def _extend_region_to_paragraph(self, region: Region):
    """Extends region dimensions based on paragraph extent"""
    if not self._has_same_origin_as_region(region):
      raise ValueError("Paragraph origin does not match with region.")

    region_extent = region.get_style(StyleProperties.Extent)

    paragraph_extent = self._paragraph.get_extent()

    # Convert extent cells to percentages
    paragraph_extent_pct = convert_cells_to_percentages(paragraph_extent, self._doc.get_cell_resolution())

    if paragraph_extent_pct.width.value > region_extent.width.value:
      # Resets the region width on the paragraph line width (up to the right of the safe area)
      # The region height always remains the same (depending on the region origin)
      region_origin: CoordinateType = region.get_style(StyleProperties.Origin)

      # Convert right cells coordinate to percentages
      right_pct = self._right * 100 / self._doc.get_cell_resolution().columns
      available_width_pct = right_pct - region_origin.x.value

      if int(paragraph_extent_pct.width.value) > available_width_pct:
        LOGGER.warning("The paragraph width overflows from the safe area (at %s)", self._paragraph.get_begin())

      max_width_pct = round(min(paragraph_extent_pct.width.value, available_width_pct))

      region_extent_pct = get_extent_from_dimensions(max_width_pct, region_extent.height.value, LengthType.Units.pct)
      region.set_style(StyleProperties.Extent, region_extent_pct)

  def _get_region_prefix(self):
    """Returns region prefix based on paragraph style"""
    if self._paragraph.get_caption_style() is SccCaptionStyle.PaintOn:
      return "paint"
    if self._paragraph.get_caption_style() is SccCaptionStyle.PopOn:
      return "pop"
    if self._paragraph.get_caption_style() is SccCaptionStyle.RollUp:
      return "rollup"
    return "region"

  def _create_matching_region(self) -> Region:
    """Creates a new region based on paragraph needs"""
    doc_regions = list(self._doc.iter_regions())

    paragraph_origin = self._paragraph.get_origin()
    region = Region(self._get_region_prefix() + str(len(doc_regions) + 1), self._doc)

    # Convert origin cells to percentages
    if self._paragraph.get_caption_style() is SccCaptionStyle.RollUp:
      # The region origin x offset
      region_origin = get_position_from_offsets(paragraph_origin.x.value, self._top)
      region_origin_pct = convert_cells_to_percentages(region_origin, self._doc.get_cell_resolution())
    else:
      # The region origin matches with paragraph origin
      region_origin_pct = convert_cells_to_percentages(paragraph_origin, self._doc.get_cell_resolution())

    region.set_style(StyleProperties.Origin, region_origin_pct)

    # The region width is initialized with he paragraph width (up to the right of the safe area)
    paragraph_extent = self._paragraph.get_extent()

    available_width = self._right - int(paragraph_origin.x.value)
    region_width = min(paragraph_extent.width.value, available_width)

    if self._paragraph.get_caption_style() is SccCaptionStyle.RollUp:
      # The region height extends from the top row to the bottom row of the safe area
      region_height = self._bottom - (self._top + 1)
      region.set_style(StyleProperties.DisplayAlign, DisplayAlignType.after)
    else:
      # The region height extends from its origin to the bottom of the safe area
      region_height = self._bottom - int(paragraph_origin.y.value) - 1
      region.set_style(StyleProperties.DisplayAlign, DisplayAlignType.before)

    region_extent = get_extent_from_dimensions(region_width, region_height)

    # Convert extent cells to percentages
    region_extent_pct = convert_cells_to_percentages(region_extent, self._doc.get_cell_resolution())
    region.set_style(StyleProperties.Extent, region_extent_pct)

    # Set default region style properties
    region.set_style(StyleProperties.ShowBackground, ShowBackgroundType.whenActive)

    self._doc.put_region(region)

    return region



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SRT paragraph"""

# pylint: disable=R0201,C0115,C0116,W0212

from fractions import Fraction
import unittest

from ttconv.srt.paragraph import SrtParagraph


class SrtParagraphTest(unittest.TestCase):

  def test_paragraph(self):
    paragraph = SrtParagraph(123)

    self.assertRaisesRegex(ValueError, "SRT paragraph begin time code must be set.", paragraph.to_string)

    paragraph.set_begin(Fraction(1234, 1))
    self.assertEqual("00:20:34,000", str(paragraph.get_begin()))

    self.assertRaisesRegex(ValueError, "SRT paragraph end time code must be set.", paragraph.to_string)

    paragraph.set_end(Fraction(1234, 1))
    self.assertEqual("00:20:34,000", str(paragraph.get_end()))

    self.assertRaisesRegex(ValueError, "SRT paragraph end time code must be greater than the begin time code.", paragraph.to_string)

    paragraph.set_end(Fraction(2345, 1))
    self.assertEqual("00:39:05,000", str(paragraph.get_end()))

    self.assertEqual("123\n00:20:34,000 --> 00:39:05,000\n", str(paragraph))
    self.assertEqual("123\n00:20:34,000 --> 00:39:05,000\n", paragraph.to_string())

    paragraph.append_text("Hello world!")

    self.assertEqual("123\n00:20:34,000 --> 00:39:05,000\nHello world!\n", str(paragraph))
    self.assertEqual("123\n00:20:34,000 --> 00:39:05,000\nHello world!\n", paragraph.to_string())

if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SCC Mid-Row Codes"""

# pylint: disable=R0201,C0115,C0116

import unittest
from typing import Optional

from ttconv.scc.codes.mid_row_codes import SccMidRowCode
from ttconv.style_properties import NamedColors, FontStyleType, TextDecorationType, ColorType

MID_ROW_CODE_VALUES = [0x1120, 0x1920, 0x1121, 0x1921, 0x1122, 0x1922, 0x1123, 0x1923, 0x1124,
                       0x1924, 0x1125, 0x1925, 0x1126, 0x1926, 0x1127, 0x1927, 0x1128, 0x1928,
                       0x1129, 0x1929, 0x112A, 0x192A, 0x112B, 0x192B, 0x112C, 0x192C, 0x112D,
                       0x192D, 0x112E, 0x192E, 0x112F, 0x192F]


class SCCMidRowCodesTest(unittest.TestCase):

  def check_mid_row_code(self, mrc, expected_value: SccMidRowCode, expected_color: Optional[ColorType],
                         expected_font_style: Optional[FontStyleType], expected_text_decoration: Optional[TextDecorationType]):
    self.assertEqual(expected_value, mrc)
    self.assertEqual(expected_color, mrc.get_color())
    self.assertEqual(expected_font_style, mrc.get_font_style())
    self.assertEqual(expected_text_decoration, mrc.get_text_decoration())

  def test_scc_mid_row_codes_valid_white(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[0]), SccMidRowCode.WHITE, NamedColors.white.value, None, None)
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[1]), SccMidRowCode.WHITE, NamedColors.white.value, None, None)

  def test_scc_mid_row_codes_valid_white_underline(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[2]), SccMidRowCode.WHITE_UNDERLINE, NamedColors.white.value,
                            None, TextDecorationType(underline=True))
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[3]), SccMidRowCode.WHITE_UNDERLINE, NamedColors.white.value,
                            None, TextDecorationType(underline=True))

  def test_scc_mid_row_codes_valid_green(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[4]), SccMidRowCode.GREEN, NamedColors.green.value, None, None)
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[5]), SccMidRowCode.GREEN, NamedColors.green.value, None, None)

  def test_scc_mid_row_codes_valid_green_underline(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[6]), SccMidRowCode.GREEN_UNDERLINE, NamedColors.green.value,
                            None, TextDecorationType(underline=True))
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[7]), SccMidRowCode.GREEN_UNDERLINE, NamedColors.green.value,
                            None, TextDecorationType(underline=True))

  def test_scc_mid_row_codes_valid_blue(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[8]), SccMidRowCode.BLUE, NamedColors.blue.value, None, None)
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[9]), SccMidRowCode.BLUE, NamedColors.blue.value, None, None)

  def test_scc_mid_row_codes_valid_blue_underline(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[10]), SccMidRowCode.BLUE_UNDERLINE, NamedColors.blue.value, None,
                            TextDecorationType(underline=True))
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[11]), SccMidRowCode.BLUE_UNDERLINE, NamedColors.blue.value, None,
                            TextDecorationType(underline=True))

  def test_scc_mid_row_codes_valid_cyan(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[12]), SccMidRowCode.CYAN, NamedColors.cyan.value, None, None)
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[13]), SccMidRowCode.CYAN, NamedColors.cyan.value, None, None)

  def test_scc_mid_row_codes_valid_cyan_underline(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[14]), SccMidRowCode.CYAN_UNDERLINE, NamedColors.cyan.value, None,
                            TextDecorationType(underline=True))
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[15]), SccMidRowCode.CYAN_UNDERLINE, NamedColors.cyan.value, None,
                            TextDecorationType(underline=True))

  def test_scc_mid_row_codes_valid_red(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[16]), SccMidRowCode.RED, NamedColors.red.value, None, None)
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[17]), SccMidRowCode.RED, NamedColors.red.value, None, None)

  def test_scc_mid_row_codes_valid_red_underline(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[18]), SccMidRowCode.RED_UNDERLINE, NamedColors.red.value, None,
                            TextDecorationType(underline=True))
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[19]), SccMidRowCode.RED_UNDERLINE, NamedColors.red.value, None,
                            TextDecorationType(underline=True))

  def test_scc_mid_row_codes_valid_yellow(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[20]), SccMidRowCode.YELLOW, NamedColors.yellow.value, None, None)
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[21]), SccMidRowCode.YELLOW, NamedColors.yellow.value, None, None)

  def test_scc_mid_row_codes_valid_yellow_underline(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[22]), SccMidRowCode.YELLOW_UNDERLINE, NamedColors.yellow.value,
                            None, TextDecorationType(underline=True))
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[23]), SccMidRowCode.YELLOW_UNDERLINE, NamedColors.yellow.value,
                            None, TextDecorationType(underline=True))

  def test_scc_mid_row_codes_valid_magenta(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[24]), SccMidRowCode.MAGENTA, NamedColors.magenta.value, None,
                            None)
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[25]), SccMidRowCode.MAGENTA, NamedColors.magenta.value, None,
                            None)

  def test_scc_mid_row_codes_valid_magenta_underline(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[26]), SccMidRowCode.MAGENTA_UNDERLINE, NamedColors.magenta.value,
                            None, TextDecorationType(underline=True))
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[27]), SccMidRowCode.MAGENTA_UNDERLINE, NamedColors.magenta.value,
                            None, TextDecorationType(underline=True))

  def test_scc_mid_row_codes_valid_italics(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[28]), SccMidRowCode.ITALICS, None, FontStyleType.italic, None)
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[29]), SccMidRowCode.ITALICS, None, FontStyleType.italic, None)

  def test_scc_mid_row_codes_valid_italics_underline(self):
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[30]), SccMidRowCode.ITALICS_UNDERLINE, None,
                            FontStyleType.italic, TextDecorationType(underline=True))
    self.check_mid_row_code(SccMidRowCode.find(MID_ROW_CODE_VALUES[31]), SccMidRowCode.ITALICS_UNDERLINE, None,
                            FontStyleType.italic, TextDecorationType(underline=True))

  def test_scc_mid_row_codes_invalid(self):
    other_code_values = [code for code in range(0x0000, 0xFFFF) if code not in MID_ROW_CODE_VALUES]

    for mrc in other_code_values:
      self.assertIsNone(SccMidRowCode.find(mrc))

if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SCC paragraph region"""

# pylint: disable=R0201,C0115,C0116,W0212

import unittest

from ttconv.model import ContentDocument, CellResolutionType
from ttconv.scc.caption_paragraph import SccCaptionParagraph, _SccParagraphRegion
from ttconv.scc.caption_style import SccCaptionStyle
from ttconv.style_properties import StyleProperties, ShowBackgroundType


class SccParagraphRegionTest(unittest.TestCase):

  def test_region_prefix(self):
    doc = ContentDocument()

    caption_paragraph = SccCaptionParagraph()
    paragraph_region = _SccParagraphRegion(caption_paragraph, doc)
    self.assertEqual("region", paragraph_region._get_region_prefix())

    caption_paragraph = SccCaptionParagraph(caption_style=SccCaptionStyle.PaintOn)
    paragraph_region = _SccParagraphRegion(caption_paragraph, doc)
    self.assertEqual("paint", paragraph_region._get_region_prefix())

    caption_paragraph = SccCaptionParagraph(caption_style=SccCaptionStyle.PopOn)
    paragraph_region = _SccParagraphRegion(caption_paragraph, doc)
    self.assertEqual("pop", paragraph_region._get_region_prefix())

    caption_paragraph = SccCaptionParagraph(caption_style=SccCaptionStyle.RollUp)
    paragraph_region = _SccParagraphRegion(caption_paragraph, doc)
    self.assertEqual("rollup", paragraph_region._get_region_prefix())

  def test_matching_region(self):
    doc = ContentDocument()
    doc_columns = 40
    doc_rows = 19
    doc.set_cell_resolution(CellResolutionType(rows=doc_rows, columns=doc_columns))

    safe_area_x_offset = 4
    safe_area_y_offset = 2

    caption_paragraph = SccCaptionParagraph(safe_area_x_offset, safe_area_y_offset)
    caption_paragraph.set_cursor_at(4, 7)
    caption_paragraph.new_caption_text()

    caption_paragraph.get_current_text().append("A 20-char long line.")

    origin = caption_paragraph.get_origin()
    self.assertEqual(11, origin.x.value)
    self.assertEqual(5, origin.y.value)

    extent = caption_paragraph.get_extent()
    self.assertEqual(20, extent.width.value)
    self.assertEqual(1, extent.height.value)

    paragraph_region = _SccParagraphRegion(caption_paragraph, doc)

    self.assertIsNone(paragraph_region._find_matching_region())
    region = paragraph_region._create_matching_region()

    self.assertEqual("region1", region.get_id())
    self.assertTrue(paragraph_region._has_same_origin_as_region(region))
    self.assertEqual(region, paragraph_region._find_matching_region())

    self.assertEqual(ShowBackgroundType.whenActive, region.get_style(StyleProperties.ShowBackground))

    region_extent = region.get_style(StyleProperties.Extent)

    self.assertEqual(50, region_extent.width.value)
    self.assertEqual(63, region_extent.height.value)

    caption_paragraph.set_cursor_at(5, 7)
    caption_paragraph.new_caption_text()
    caption_paragraph.get_current_text().append("This is another 34-char long line.")

    origin = caption_paragraph.get_origin()
    self.assertEqual(11, origin.x.value)
    self.assertEqual(5, origin.y.value)

    extent = caption_paragraph.get_extent()
    self.assertEqual(34, extent.width.value)
    self.assertEqual(2, extent.height.value)

    paragraph_region = _SccParagraphRegion(caption_paragraph, doc)

    self.assertEqual(region, paragraph_region._find_matching_region())
    paragraph_region._extend_region_to_paragraph(region)
    self.assertTrue(paragraph_region._has_same_origin_as_region(region))

    region_extent = region.get_style(StyleProperties.Extent)

    self.assertEqual(62, region_extent.width.value)
    self.assertEqual(63, region_extent.height.value)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the configuration module"""

# pylint: disable=R0201,C0115,C0116,W0212
import json
import unittest
from fractions import Fraction

from ttconv.config import GeneralConfiguration
from ttconv.imsc.config import IMSCWriterConfiguration
from ttconv.imsc.attributes import TimeExpressionSyntaxEnum
from ttconv.isd import ISDConfiguration
from ttconv.scc.config import SccReaderConfiguration, TextAlignment
from ttconv.tt import CONFIGURATIONS


class ConfigurationTest(unittest.TestCase):

  def test_config_parsing(self):

    config_json = """{
      "general": {
        "progress_bar": false,
        "log_level": "INFO",
        "document_lang" : "es-419"
      },
      "imsc_writer": {
        "fps": "30000/1001",
        "time_format": "frames"
      },
      "isd" : {
        "multi_thread": false
      },
      "imsc_reader" : {},
      "srt_writer" : {},
      "scc_reader" : {
        "text_align": "right"
      }
    }
    """
    config_dict = json.loads(config_json)

    expected_configurations = [
      GeneralConfiguration(log_level='INFO', progress_bar=False, document_lang="es-419"),
      IMSCWriterConfiguration(time_format=TimeExpressionSyntaxEnum.frames, fps=Fraction(30000, 1001)),
      ISDConfiguration(multi_thread=False),
      SccReaderConfiguration(text_align=TextAlignment.RIGHT)
    ]

    module_configurations = []
    for config_class in CONFIGURATIONS:
      config_value = config_dict.get(config_class.name())

      if config_value is None:
        continue

      module_config = config_class.parse(config_value)

      self.assertIsNotNone(module_config)
      module_configurations.append(module_config)

    for exp_config in expected_configurations:
      self.assertTrue(exp_config in module_configurations)

  def test_default_config_parsing(self):

    config_json = """{
      "imsc_writer": {
        "fps": "30000/1001",
        "time_format": "frames"
      },
      "general": {},
      "isd" : {},
      "imsc_reader" : {},
      "srt_writer" : {},
      "scc_reader" : {}
    }
    """
    config_dict = json.loads(config_json)

    expected_configurations = [
      GeneralConfiguration(),
      IMSCWriterConfiguration(time_format=TimeExpressionSyntaxEnum.frames, fps=Fraction(30000, 1001)),
      ISDConfiguration(),
      SccReaderConfiguration()
    ]

    module_configurations = []

    for config_class in CONFIGURATIONS:
      config_value = config_dict.get(config_class.name())

      if config_value is None:
        continue

      module_config = config_class.parse(config_value)

      self.assertIsNotNone(module_config)
      module_configurations.append(module_config)

    for exp_config in expected_configurations:
      self.assertTrue(exp_config in module_configurations)

  def test_empty_config_parsing(self):

    config_dict = json.loads('{}')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2021, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""WebVTT writer"""

import logging
from fractions import Fraction
from typing import Dict, List, Optional

import ttconv.model as model
from ttconv.vtt.config import VTTWriterConfiguration
import ttconv.vtt.style as style
from ttconv.filters.isd.default_style_properties import DefaultStylePropertyValuesISDFilter
from ttconv.filters.isd.merge_paragraphs import ParagraphsMergingISDFilter
from ttconv.filters.isd.merge_regions import RegionsMergingISDFilter
from ttconv.filters.isd.supported_style_properties import SupportedStylePropertiesISDFilter
from ttconv.isd import ISD
from ttconv.vtt.cue import VttCue
from ttconv.vtt.css_class import CssClass
from ttconv.style_properties import DirectionType, ExtentType, PositionType, StyleProperties, FontStyleType, NamedColors, \
                                    FontWeightType, TextDecorationType, DisplayAlignType, TextAlignType

LOGGER = logging.getLogger(__name__)


class VttContext:
  """VTT writer context"""

  def __init__(self, config: VTTWriterConfiguration):
    self._captions_counter: int = 0
    self._begin: Fraction = Fraction(0)
    self._end: Fraction = Fraction(0)
    self._paragraphs: List[VttCue] = []
    self._css_classes: List[CssClass] = []
    self._colors_used: Dict[str, str] = {}
    self._background_colors_used: Dict[str, str] = {}
    self._config = config

    self._filters = []

    if not self._config.line_position:
      self._filters.append(RegionsMergingISDFilter())

    self._filters.append(ParagraphsMergingISDFilter())

    supported_styles = {
      StyleProperties.FontWeight: [],
      StyleProperties.FontStyle: [
        FontStyleType.normal,
        FontStyleType.italic
      ],
      StyleProperties.TextDecoration: [
        TextDecorationType.underline
      ],
      StyleProperties.Color: [],
      StyleProperties.BackgroundColor: []
    }

    if self._config.line_position:
      supported_styles.update({
        StyleProperties.Position: [],
        StyleProperties.Origin: [],
        StyleProperties.DisplayAlign: [],
        StyleProperties.Extent: [],
      })

    if self._config.text_align:
      supported_styles.update({
        StyleProperties.TextAlign: [],
        StyleProperties.Direction: [],
      })

    self._filters.append(SupportedStylePropertiesISDFilter(supported_styles))

    self._filters.append(
      DefaultStylePropertyValuesISDFilter({
        StyleProperties.Color: NamedColors.white.value,
        StyleProperties.BackgroundColor: NamedColors.transparent.value,
        StyleProperties.FontWeight: FontWeightType.normal,
        StyleProperties.FontStyle: FontStyleType.normal,
      })
    )

  def process_inline_element(self, element: model.ContentElement, begin: Fraction, end: Optional[Fraction]):
    """Converts inline element (span and br) to VTT content"""

    if isinstance(element, model.Span):
      is_bold = style.is_element_bold(element)
      is_italic = style.is_element_italic(element)
      is_underlined = style.is_element_underlined(element)
      color = style.get_color(element)
      bg_color = style.get_background_color(element)

      if color is not None:
        if self._colors_used.get(color) is None:
          color_classname = style.get_color_classname(color)
          self._colors_used[color] = color_classname
          self._css_classes.append(CssClass("color", color, color_classname))
        else:
          color_classname = self._colors_used[color]
        self._paragraphs[-1].append_text(style.COLOR_TAG_IN.format(color_classname))

      if bg_color is not None:
        if self._background_colors_used.get(bg_color) is None:
          bg_color_classname = style.get_background_color_classname(bg_color)
          self._background_colors_used[bg_color] = bg_color_classname
          self._css_classes.append(CssClass("background-color", bg_color, bg_color_classname))
        else:
          bg_color_classname = self._background_colors_used[bg_color]
        self._paragraphs[-1].append_text(style.BG_COLOR_TAG_IN.format(bg_color_classname))

      if is_bold:
        self._paragraphs[-1].append_text(style.BOLD_TAG_IN)
      if is_italic:
        self._paragraphs[-1].append_text(style.ITALIC_TAG_IN)
      if is_underlined:
        self._paragraphs[-1].append_text(style.UNDERLINE_TAG_IN)

      for elem in list(element):
        self.process_inline_element(elem, begin, end)

      if is_underlined:
        self._paragraphs[-1].append_text(style.UNDERLINE_TAG_OUT)
      if is_italic:
        self._paragraphs[-1].append_text(style.ITALIC_TAG_OUT)
      if is_bold:
        self._paragraphs[-1].append_text(style.BOLD_TAG_OUT)
      if color is not None:
        self._paragraphs[-1].append_text(style.COLOR_TAG_OUT)
      if bg_color is not None:
        self._paragraphs[-1].append_text(style.BG_COLOR_TAG_OUT)

    if isinstance(element, model.Br):
      self._paragraphs[-1].append_text("\n")

    if isinstance(element, model.Text):
      self._paragraphs[-1].append_text(element.get_text())

  def process_p(self, region: ISD.Region, element: model.P, begin: Fraction, end: Optional[Fraction]):
    """Process p element"""

    self._captions_counter += 1

    cue = VttCue(self._captions_counter if self._config.cue_id else None)
    cue.set_begin(begin)
    cue.set_end(end)

    if self._config.line_position:
      display_align = region.get_style(StyleProperties.DisplayAlign)
      position: PositionType = region.get_style(StyleProperties.Position)
      extent: ExtentType = region.get_style(StyleProperties.Extent)
      
      if display_align == DisplayAlignType.after:
        cue.set_line(round(position.v_offset.value + extent.height.value))
        cue.set_align(VttCue.LineAlignment.end)
      elif display_align == DisplayAlignType.before:
        cue.set_line(round(position.v_offset.value))
        cue.set_align(VttCue.LineAlignment.start)
      else:
        cue.set_line(round(position.v_offset.value + extent.height.value / 2))
        cue.set_align(VttCue.LineAlignment.center)

    if self._config.text_align:
      direction = element.get_style(StyleProperties.Direction)
      text_align = element.get_style(StyleProperties.TextAlign)
      if text_align == TextAlignType.center:
        cue.set_textalign(VttCue.TextAlignment.center)
      elif text_align == TextAlignType.start:
        cue.set_textalign(VttCue.TextAlignment.right if direction == DirectionType.rtl else VttCue.TextAlignment.left)
      elif text_align == TextAlignType.end:
        cue.set_textalign(VttCue.TextAlignment.left if direction == DirectionType.rtl else VttCue.TextAlignment.right)

    self._paragraphs.append(cue)

    for elem in list(element):
      self.process_inline_element(elem, begin, end)
    
    self._paragraphs[-1].normalize_eol()

    if self._paragraphs[-1].is_only_whitespace_or_empty():
      LOGGER.debug("Removing empty paragraph.")
      self._paragraphs.pop()
      self._captions_counter -= 1

  def add_isd(self, isd: ISD, begin: Fraction, end: Optional[Fraction]):
    """Converts and appends ISD content to VTT content"""

    LOGGER.debug(
      "Append ISD from %ss to %ss to VTT content.",
      float(begin),
      float(end) if end is not None else "unbounded"
    )

    # filter the ISD to remove unsupported features

    for vtt_filter in self._filters:
      vtt_filter.process(isd)


    # process the ISD regions

    is_isd_empty = True

    for region in isd.iter_regions():

      if len(region) > 0:
        is_isd_empty = False

      for body in region:
        for div in list(body):
          for p in list(div):
            self.process_p(region, p, begin, end)

    if is_isd_empty:
      LOGGER.debug("Skipping empty paragraph.")

  def finish(self):
    """Checks and processes the last paragraph"""

    LOGGER.debug("Check and process the last VTT paragraph.")

    if self._paragraphs and self._paragraphs[-1].get_end() is None:
      if self._paragraphs[-1].is_only_whitespace_or_empty():
        # if the last paragraph contains only whitespace, remove it
        LOGGER.debug("Removing empty unbounded last paragraph.")
        self._paragraphs.pop()

      else:
        # set default end time code
        LOGGER.warning("Set a default end value to paragraph (begin + 10s).")
        self._paragraphs[-1].set_end(self._paragraphs[-1].get_begin().to_seconds() + 10.0)

  def style_block(self):
    """Generated CSS INLINE STYLE Block"""
    style_block = ""
    cue_default = "\n".join(("::cue {","  background-color: transparent;", "}\n"))
    if len(self._css_classes) > 0:
      style_block = "STYLE\n" + cue_default +"\n".join(css.to_string() for css in self._css_classes) + "\n\n"
    return style_block

  def __str__(self) -> str:
    return "WEBVTT\n\n" + self.style_block() + "\n".join(p.to_string() for p in self._paragraphs)


#
# vtt writer
#


def from_model(doc: model.ContentDocument, config = None, progress_callback=lambda _: None) -> str:
  """Converts the data model to a VTT document"""

  # split progress between ISD construction and VTT writing
  def _isd_progress(progress: float):
    progress_callback(progress / 2)

  # create context
  vtt = VttContext(config if config is not None else VTTWriterConfiguration())

  # Compute ISDs
  isds = ISD.generate_isd_sequence(doc, _isd_progress)

  # process ISDs
  for i, (begin, isd) in enumerate(isds):

    end = isds[i + 1][0] if i + 1 < len(isds) else None

    vtt.add_isd(isd, begin, end)

    progress_callback(0.5 + (i + 1) / len(isds) / 2)

  vtt.finish()

  return str(vtt)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SCC Special characters"""

# pylint: disable=R0201,C0115,C0116,W0212

import unittest

from ttconv.scc.codes.characters import unicode_to_scc
from ttconv.scc.codes.extended_characters import SccExtendedCharacter
from ttconv.scc.codes.special_characters import SccSpecialCharacter


class SccSpecialCharactersTest(unittest.TestCase):

  def test_unicode_to_scc_special_character(self):
    for spec_char in list(SccSpecialCharacter):
      unicode_char = spec_char.get_unicode_value()
      with self.subTest(unicode_char=unicode_char):
        # skip the TRANSPARENT_SPACE character
        if unicode_char == " ":
          continue
        self.assertEqual(int.from_bytes(unicode_to_scc(unicode_char), byteorder='big'), spec_char.get_ch1_value())


  def test_scc_special_character_values(self):
    special_char_codes = list(range(0x1130, 0x1140)) + list(range(0x1930, 0x1940))

    extended_char_codes = list(range(0x1220, 0x1240)) + list(range(0x1320, 0x1340)) + \
                          list(range(0x1A20, 0x1A40)) + list(range(0x1B20, 0x1B40))

    for code in special_char_codes:
      self.assertIsNotNone(SccSpecialCharacter.find(code))

    for code in extended_char_codes:
      self.assertIsNotNone(SccExtendedCharacter.find(code))

  def test_scc_special_characters_unicode_values(self):
    spec_char = SccSpecialCharacter.find(0x1130)
    self.assertEqual(SccSpecialCharacter.REGISTERED_MARK_SYMBOL, spec_char)
    self.assertEqual('\u00AE', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1930)
    self.assertEqual(SccSpecialCharacter.REGISTERED_MARK_SYMBOL, spec_char)
    self.assertEqual('\u00AE', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1131)
    self.assertEqual(SccSpecialCharacter.DEGREE_SIGN, spec_char)
    self.assertEqual('\u00B0', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1931)
    self.assertEqual(SccSpecialCharacter.DEGREE_SIGN, spec_char)
    self.assertEqual('\u00B0', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1132)
    self.assertEqual(SccSpecialCharacter.VULGAR_FRACTION_ONE_HALF, spec_char)
    self.assertEqual('\u00BD', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1932)
    self.assertEqual(SccSpecialCharacter.VULGAR_FRACTION_ONE_HALF, spec_char)
    self.assertEqual('\u00BD', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1133)
    self.assertEqual(SccSpecialCharacter.INVERTED_QUESTION_MARK, spec_char)
    self.assertEqual('\u00BF', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1933)
    self.assertEqual(SccSpecialCharacter.INVERTED_QUESTION_MARK, spec_char)
    self.assertEqual('\u00BF', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1134)
    self.assertEqual(SccSpecialCharacter.TRADEMARK_SYMBOL, spec_char)
    self.assertEqual('\u2122', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1934)
    self.assertEqual(SccSpecialCharacter.TRADEMARK_SYMBOL, spec_char)
    self.assertEqual('\u2122', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1135)
    self.assertEqual(SccSpecialCharacter.CENTS_SIGN, spec_char)
    self.assertEqual('\u00A2', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1935)
    self.assertEqual(SccSpecialCharacter.CENTS_SIGN, spec_char)
    self.assertEqual('\u00A2', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1136)
    self.assertEqual(SccSpecialCharacter.POUNDS_STERLING_SIGN, spec_char)
    self.assertEqual('\u00A3', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1936)
    self.assertEqual(SccSpecialCharacter.POUNDS_STERLING_SIGN, spec_char)
    self.assertEqual('\u00A3', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1137)
    self.assertEqual(SccSpecialCharacter.MUSIC_NOTE, spec_char)
    self.assertEqual('\u266A', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1937)
    self.assertEqual(SccSpecialCharacter.MUSIC_NOTE, spec_char)
    self.assertEqual('\u266A', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1138)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_A_WITH_GRAVE_ACCENT, spec_char)
    self.assertEqual('\u00E0', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1938)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_A_WITH_GRAVE_ACCENT, spec_char)
    self.assertEqual('\u00E0', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1139)
    self.assertEqual(SccSpecialCharacter.TRANSPARENT_SPACE, spec_char)
    self.assertEqual('\u0020', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x1939)
    self.assertEqual(SccSpecialCharacter.TRANSPARENT_SPACE, spec_char)
    self.assertEqual('\u0020', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x113A)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_E_WITH_GRAVE_ACCENT, spec_char)
    self.assertEqual('\u00E8', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x193A)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_E_WITH_GRAVE_ACCENT, spec_char)
    self.assertEqual('\u00E8', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x113B)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_A_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00E2', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x193B)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_A_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00E2', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x113C)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_E_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00EA', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x193C)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_E_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00EA', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x113D)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_I_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00EE', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x193D)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_I_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00EE', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x113E)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_O_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00F4', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x193E)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_O_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00F4', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x113F)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_U_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00FB', spec_char.get_unicode_value())

    spec_char = SccSpecialCharacter.find(0x193F)
    self.assertEqual(SccSpecialCharacter.LOWER_CASE_U_WITH_CIRCUMFLEX, spec_char)
    self.assertEqual('\u00FB', spec_char.get_unicode_value())


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import unittest
from fractions import Fraction
from ttconv.utils import DisjointIntervals

class DisjointIntervalsTest(unittest.TestCase):

  def test_add_simple(self):
    di = DisjointIntervals()
    di.add(Fraction(0), Fraction(10))
    self.assertEqual(len(di), 1)
    self.assertTrue(di.contains(Fraction(0)))
    self.assertTrue(di.contains(Fraction(5)))
    self.assertFalse(di.contains(Fraction(10)))

  def test_add_disjoint(self):
    di = DisjointIntervals()
    di.add(Fraction(0), Fraction(10))
    di.add(Fraction(20), Fraction(30))
    self.assertEqual(len(di), 2)
    self.assertTrue(di.contains(Fraction(5)))
    self.assertTrue(di.contains(Fraction(25)))
    self.assertFalse(di.contains(Fraction(15)))

  def test_add_overlap(self):
    di = DisjointIntervals()
    di.add(Fraction(0), Fraction(10))
    di.add(Fraction(5), Fraction(15))
    self.assertEqual(len(di), 1)
    self.assertEqual(list(di)[0], (Fraction(0), Fraction(15)))

  def test_add_adjacent(self):
    di = DisjointIntervals()
    di.add(Fraction(0), Fraction(10))
    di.add(Fraction(10), Fraction(20))
    self.assertEqual(len(di), 1)
    self.assertEqual(list(di)[0], (Fraction(0), Fraction(20)))

  def test_add_contained(self):
    di = DisjointIntervals()
    di.add(Fraction(0), Fraction(20))
    di.add(Fraction(5), Fraction(15))
    self.assertEqual(len(di), 1)
    self.assertEqual(list(di)[0], (Fraction(0), Fraction(20)))

  def test_add_enclosing(self):
    di = DisjointIntervals()
    di.add(Fraction(5), Fraction(15))
    di.add(Fraction(0), Fraction(20))
    self.assertEqual(len(di), 1)
    self.assertEqual(list(di)[0], (Fraction(0), Fraction(20)))

  def test_add_merge_multiple(self):
    di = DisjointIntervals()
    di.add(Fraction(0), Fraction(10))
    di.add(Fraction(20), Fraction(30))
    di.add(Fraction(40), Fraction(50))
    di.add(Fraction(5), Fraction(45))
    self.assertEqual(len(di), 1)
    self.assertEqual(list(di)[0], (Fraction(0), Fraction(50)))

  def test_infinity_end(self):
    di = DisjointIntervals()
    di.add(Fraction(10), None)
    self.assertEqual(len(di), 1)
    self.assertEqual(list(di)[0], (Fraction(10), None))
    self.assertTrue(di.contains(Fraction(1000)))

  def test_merge_infinity(self):
    di = DisjointIntervals()
    di.add(Fraction(0), Fraction(10))
    di.add(Fraction(5), None)
    self.assertEqual(len(di), 1)
    self.assertEqual(list(di)[0], (Fraction(0), None))

  def test_merge_infinity_gap(self):
    di = DisjointIntervals()
    di.add(Fraction(0), Fraction(10))
    di.add(Fraction(20), None)
    di.add(Fraction(5), Fraction(25))
    self.assertEqual(len(di), 1)
    self.assertEqual(list(di)[0], (Fraction(0), None))

  def test_add_invalid(self):
    di = DisjointIntervals()
    with self.assertRaises(ValueError):
      di.add(Fraction(10), Fraction(5))
    with self.assertRaises(ValueError):
      di.add(Fraction(10), Fraction(10))

if __name__ == '__main__':
  unittest.main()